package com.diagbot.client;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.client.hystrix.CdssCoreHystrix;
import com.diagbot.dto.ConceptDetailDTO;
import com.diagbot.dto.ConceptScaleDTO;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.GetAllForRelationDTO;
import com.diagbot.dto.IndexBatchDTO;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.KlConceptStaticDTO;
import com.diagbot.dto.KlRuleByIdParDTO;
import com.diagbot.dto.KlRuleInfoDTO;
import com.diagbot.dto.KllisDetailDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.PushPlanDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.dto.StaticKnowledgeIndexPageDTO;
import com.diagbot.dto.TermConceptDTO;
import com.diagbot.entity.KlRuleMenuWrapper;
import com.diagbot.vo.*;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description: CDSS核心类客户端
 * @author: gaodm
 * @time: 2020/7/28 9:29
 */
@FeignClient(name = "cdss-core", url = "${cdss-core.url}", fallback = CdssCoreHystrix.class)
public interface CdssCoreClient {

    @PostMapping("/core/indication")
    RespDTO<IndicationDTO> indication(@Valid @RequestBody IndicationPushVO indicationPushVO);

    @PostMapping("/core/push")
    RespDTO<PushDTO> push(@RequestBody @Valid PushVO pushVo);

    /**
     * 推送持续检验检查计划
     */
    @PostMapping("/core/pushPlan")
    RespDTO<PushPlanDTO> pushPlan(@RequestBody PushPlanVO pushPlanVO);

    /**
     * 化验大项、化验小项、辅检、诊断、药品、手术等检索
     *
     * @param retrievalVO
     * @return
     */
    @PostMapping("/retrieval/index")
    RespDTO<RetrievalDTO> index(@RequestBody @Valid RetrievalVO retrievalVO);

    /**
     * 国药准字匹配
     *
     * @param indexByApprovalVO
     * @return
     */
    @PostMapping("/retrieval/indexByApproval")
    RespDTO<List<IndexBatchDTO>> indexByApproval(@Valid @RequestBody IndexByApprovalVO indexByApprovalVO);

    /**
     * 静态知识检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    @PostMapping("/retrieval/staticKnowledgeIndex")
    RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndex(@RequestBody @Valid StaticKnowledgeIndexVO staticKnowledgeIndexVO);

    /**
     * 术语关联医学术语检索
     *
     * @param filterVO
     * @return
     */
    @PostMapping("/retrieval/filter")
    RespDTO<List<IndexBatchDTO>> filter(@Valid @RequestBody FilterVO filterVO);

    /**
     * 更新是否有静态知识状态
     *
     * @param hasStaticKnowledgeVO
     * @return
     */
    @PostMapping("/staticKnowledge/updateHasInfoStatus")
    RespDTO<Boolean> updateHasInfoStatus(@Valid @RequestBody HasStaticKnowledgeVO hasStaticKnowledgeVO);

    /**
     * 术语批量校验
     *
     * @param conceptVO
     * @return
     */
    @PostMapping("/retrieval/getConceptNames")
    RespDTO<List<IndexBatchDTO>> getConceptNames(@Valid @RequestBody ConceptVO conceptVO);


    /**
     * 化验小项术语批量查询获取范围
     *
     * @param kllisDetailVO
     * @return
     */
    @PostMapping("/retrieval/getLisDetaisByNames")
    RespDTO<Map<String, KllisDetailDTO>> getLisDetaisByNames(@RequestBody KllisDetailVO kllisDetailVO);

    /**
     * 取上级静态知识
     *
     * @param staticKnowledgeNameVO
     * @return
     */
    @PostMapping("/staticKnowledge/getSuperName")
    RespDTO<StaticKnowledgeNameVO> getSuperName(@Valid @RequestBody StaticKnowledgeNameVO staticKnowledgeNameVO);

    /**
     * 取上级静态知识（批量）
     *
     * @param staticKnowledgeNameVO
     * @return
     */
    @PostMapping("/staticKnowledge/getSuperNameBatch")
    RespDTO<List<StaticKnowledgeNameVO>> getSuperNameBatch(@Valid @RequestBody List<StaticKnowledgeNameVO> staticKnowledgeNameVO);

    /**
     * 根据名称和类型获取静态知识内容
     *
     * @param scaleStaticAllVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/getStaticKnowledge")
    RespDTO<StaticKnowledgeDTO> getStaticKnowledge(@Valid @RequestBody ScaleStaticAllVO scaleStaticAllVO);

    /**
     * 分页查询
     *
     * @param klConceptStaticPageVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/getPage")
    RespDTO<Page<KlConceptStaticDTO>> getPage(@Valid @RequestBody KlConceptStaticPageVO klConceptStaticPageVO);


    /**
     * 静态知识检索（分页返回）
     *
     * @param staticKnowledgeIndexPageVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/staticIndexPage")
    RespDTO<Page<StaticKnowledgeIndexPageDTO>> staticIndexPage(@Valid @RequestBody StaticKnowledgeIndexPageVO staticKnowledgeIndexPageVO);

    /**
     * 保存静态知识
     *
     * @param klConceptStaticVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/saveOrUpdateRecord")
    RespDTO<Boolean> saveOrUpdateRecord(@Valid @RequestBody KlConceptStaticVO klConceptStaticVO);

    /**
     * 静态知识启用禁用
     *
     * @param changeStatusVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/changeStatus")
    RespDTO<Boolean> changeStatus(@Valid @RequestBody ChangeStatusVO changeStatusVO);

    /**
     * 根据id获取静态知识
     *
     * @param idVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/getRecordById")
    RespDTO<KlConceptStaticDTO> getRecordById(@Valid @RequestBody IdVO idVO);

    /**
     * 静态知识是否已存在
     *
     * @param klConceptStaticVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/isExist")
    RespDTO<Boolean> isExist(@Valid @RequestBody KlConceptStaticVO klConceptStaticVO);

    /**
     * 推理结果匹配静态知识
     *
     * @param getDetailVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/getDetailByConcept")
    RespDTO<Map<String, List<ConceptDetailDTO>>> getDetailByConcept(@Valid @RequestBody GetDetailVO getDetailVO);

    /**
     * 推理结果匹配量表
     *
     * @param getDetailVO
     * @return
     */
    @PostMapping("/kl/conceptStatic/getScaleDetailByConcept")
    RespDTO<Map<String, ConceptScaleDTO>> getScaleDetailByConcept(@RequestBody @Valid GetDetailVO getDetailVO);

    /**
     * 获取规则下拉菜单信息
     *
     * @param klRuleMenuVO
     * @return
     */
    @PostMapping("/klRulePlan/getMenu")
    RespDTO<List<KlRuleMenuWrapper>> getMenus(@RequestBody KlRuleMenuVO klRuleMenuVO);

    //"分页获取规则维护列表
    @PostMapping("/klRule/getKlRuleInfoPage")
    RespDTO<Page<KlRuleInfoDTO>> getKlRuleInfoPages(@RequestBody KlRuleInfoVO klRuleInfoVO);


    //根据规则Id获取规则详情
    @PostMapping("/klRule/getByIdRuleInfo")
    RespDTO<KlRuleByIdParDTO> getByIdRuleInfoAll(@RequestBody @Valid KlRuleByIdVO klRuleByIdVO);

    //保存规则详情[
    @PostMapping("/klRule/saveRuleInfo")
    RespDTO<Boolean> saveRuleInfoAll(@RequestBody @Valid KlRuleInfoSaveVO klRuleInfoSaveVO);

    //刪除规则详情
    @PostMapping("/klRule/clearRuleInfo")
    RespDTO<Boolean> clearRuleInfoAll(@RequestBody @Valid KlRuleInfoClearVO klRuleInfoClearVO);

    //停用规则
    @PostMapping("/klRule/disableRuleInfo")
    RespDTO<Boolean> disableRuleInfos(@RequestBody @Valid KlRuleSatartOrdisaVO klRuleSatartOrdisaVO);

    //启用规则
    @PostMapping("/klRule/startRuleInfo")
    RespDTO<Boolean> startRuleInfos(@RequestBody @Valid KlRuleSatartOrdisaVO klRuleSatartOrdisaVO);

    @PostMapping("/kl/dictionary/getDictionaryInfo")
    RespDTO<Map<Long, List<DictionaryInfoDTO>>> getDictionaryAll();

    @PostMapping("/klDisease/searchConcept")
    RespDTO<List<GetAllForRelationDTO>> searchConcept(@Valid @RequestBody SearchConceptVO searchConceptVO);

    //术语集合新增
    @PostMapping("/klDisease/addConceptClass")
    RespDTO<List<GetAllForRelationDTO>> addConceptClass(@Valid @RequestBody SearchConceptVO searchConceptVO);

    //规则维护术语查询
    @PostMapping("/klDisease/searchConceptRuleClass")
    RespDTO<List<GetAllForRelationDTO>> searchConceptRuleClass(@Valid @RequestBody SearchConceptVO searchConceptVO);

    @PostMapping("/cache/clearRuleInfoAll")
    RespDTO<Boolean> clearRuleAll();

    @PostMapping("/term/termMatching")
    public RespDTO<List<TermConceptDTO>> getTermMatching(@Valid @RequestBody TermMatchingVO termMatchingVO);
}
