package com.diagbot.util;

import com.diagbot.dto.RespDTO;
import com.diagbot.entity.TranLog;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.vo.CdssLogBaseVO;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: Cdss日志工具类
 * @author: gaodm
 * @time: 2021/10/20 10:30
 */
public class CdssLogUtil {

    private final static String CDSS_REQ = "Cdss_Req";
    private final static String CDSS_RESP = "Cdss_Resp";
    //    private final static String CDSS_HOSPITAL_ID = "Cdss_Hospital_Id";
    //    private final static String CDSS_SUB_HOSPITAL_NAME = "Cdss_Sub_Hospital_Name";
    //    private final static String CDSS_SUB_HOSPITAL_CODE = "Cdss_Sub_Hospital_Code";
    //    private final static String CDSS_PATIENT_ID = "Cdss_Patient_Id";
    private final static String CDSS_LOG_SP = "→";

    /**
     * 入参设置
     *
     * @param joinPoint
     */
    public static TranLog tranLogReqAspect(ProceedingJoinPoint joinPoint) {
        TranLog tranLog = new TranLog();
        //请求的参数
        Object[] args = joinPoint.getArgs();
        //请求的参数
        String params = "";
        for (Object o : args) {
            params += FastJsonUtils.getBeanToJson(o);
            break;
        }
        if (!StringUtil.isEmpty(params)) {
            tranLog.setParams(params);
        }
        return tranLog;
    }

    /**
     * 正常结束出参
     *
     * @param tranLog
     * @param joinPoint
     * @return
     * @throws Throwable
     */
    public static TranLog tranLogRespAspect(Object proceed, TranLog tranLog, long start, ProceedingJoinPoint joinPoint) throws Throwable {
        //请求的参数
        Object[] args = joinPoint.getArgs();
        for (Object o : args) {
            //第一个参数继承基类
            if (o instanceof CdssLogBaseVO) {
                CdssLogBaseVO p = (CdssLogBaseVO) o;
                tranLog.setPatientId(p.getPatientId());
                tranLog.setHospitalId(p.getHospitalId());
                tranLog.setSubHospitalCode(p.getSubHospitalCode());
                tranLog.setSubHospitalName(p.getSubHospitalName());
                tranLog.setSceneName(p.getSceneName());
            }
            break;
        }
        //出参日志记录
        String result = "";
        result = FastJsonUtils.getBeanToJson(proceed);
        tranLog.setSuccessFlag(1);
        if (!StringUtil.isEmpty(result)) {
            tranLog.setResult(result);
            if (proceed instanceof RespDTO) {
                RespDTO respDTO = (RespDTO) proceed;
                if (!respDTO.code.equals("0")) {
                    tranLog.setSuccessFlag(0);
                }
            }
        }
        long execTime = System.currentTimeMillis() - start;
        tranLog.setGmtResp(DateUtil.now());
        tranLog.setExecTime(String.valueOf(execTime));
        //业务处理
        bizDeal(tranLog);
        return tranLog;
    }

    /**
     * 异常结束
     *
     * @param joinPoint
     * @param ex
     * @return
     */
    public static TranLog tranLogRespAspectThrow(JoinPoint joinPoint, TranLog tranLog, long start, Throwable ex) {
        //请求的参数
        Object[] args = joinPoint.getArgs();
        //请求的参数
        String params = "";
        for (Object o : args) {
            params += FastJsonUtils.getBeanToJson(o);
            //第一个参数继承基类
            if (o instanceof CdssLogBaseVO) {
                CdssLogBaseVO p = (CdssLogBaseVO) o;
                tranLog.setPatientId(p.getPatientId());
                tranLog.setHospitalId(p.getHospitalId());
                tranLog.setSubHospitalCode(p.getSubHospitalCode());
                tranLog.setSubHospitalName(p.getSubHospitalName());
                tranLog.setSceneName(p.getSceneName());
            }
            break;
        }
        if (!StringUtil.isEmpty(params)) {
            tranLog.setParams(params);
        }
        //出参设置
        String result = "";
        result = FastJsonUtils.getBeanToJson(handleException((Exception) ex));
        tranLog.setResult(result);
        tranLog.setSuccessFlag(0);
        long execTime = System.currentTimeMillis() - start;
        tranLog.setGmtResp(DateUtil.now());
        tranLog.setExecTime(String.valueOf(execTime));
        //业务处理
        bizDeal(tranLog);
        return tranLog;
    }

    /**
     * 抛错信息处理
     *
     * @param e
     * @return 结果参数
     */
    private static RespDTO handleException(Exception e) {
        RespDTO resp = new RespDTO();
        if (e instanceof BindException) {
            BindException ex = (BindException) e;
            Map<String, String> stringMap = new HashMap<>();
            for (FieldError fieldError : ex.getBindingResult().getFieldErrors()) {
                stringMap.put(fieldError.getField(), fieldError.getDefaultMessage());
            }
            String msg = FastJsonUtils.getBeanToJson(stringMap);
            resp.code = CommonErrorCode.PARAM_ERROR.getCode();
            resp.msg = msg;
            return resp;
        }
        if (e instanceof MethodArgumentNotValidException) {
            MethodArgumentNotValidException ex = (MethodArgumentNotValidException) e;
            Map<String, String> stringMap = new HashMap<>();
            for (FieldError fieldError : ex.getBindingResult().getFieldErrors()) {
                stringMap.put(fieldError.getField(), fieldError.getDefaultMessage());
            }
            String msg = FastJsonUtils.getBeanToJson(stringMap);
            resp.code = CommonErrorCode.PARAM_ERROR.getCode();
            resp.msg = msg;
            return resp;
        }
        if (e instanceof MissingServletRequestParameterException) {
            MissingServletRequestParameterException ex = (MissingServletRequestParameterException) e;
            Map<String, String> stringMap = new HashMap<>();
            stringMap.put(ex.getParameterName(), "不能为null");
            String msg = FastJsonUtils.getBeanToJson(stringMap);
            resp.code = CommonErrorCode.PARAM_ERROR.getCode();
            resp.msg = msg;
            return resp;
        }
        if (e instanceof CommonException) {
            CommonException taiChiException = (CommonException) e;
            resp.code = taiChiException.getCode();
            resp.msg = e.getMessage();
            return resp;
        }
        resp.code = CommonErrorCode.FAIL.getCode();
        resp.msg = e.getMessage();
        return resp;
    }

    /**
     * 日志处理
     *
     * @param tranLog
     */
    private static void bizDeal(TranLog tranLog) {
        HttpServletResponse response = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getResponse();
        //入参处理
        String params = tranLog.getParams();
        String req = response.getHeader(CDSS_REQ);
        if (StringUtil.isNotBlank(req)) {
            if (StringUtil.isNotBlank(params)) {
                params += CDSS_LOG_SP + req;
            } else {
                params = req;
            }
            response.setHeader(CDSS_REQ, "");
        }
        tranLog.setParams(params);
        //出参处理
        String result = tranLog.getResult();
        String resp = response.getHeader(CDSS_RESP);
        if (StringUtil.isNotBlank(resp)) {
            if (StringUtil.isNotBlank(result)) {
                result += CDSS_LOG_SP + resp;
            } else {
                result = resp;
            }
            response.setHeader(CDSS_RESP, "");
        }
        tranLog.setResult(result);

        //todo 个性化处理

        //        //设置医院id
        //        Long hospitalId = tranLog.getHospitalId();
        //        String hosId = response.getHeader(CDSS_HOSPITAL_ID);
        //        if (StringUtil.isNotBlank(hosId)) {
        //            hospitalId = Long.valueOf(hosId);
        //            response.setHeader(CDSS_HOSPITAL_ID, "");
        //        }
        //        tranLog.setHospitalId(hospitalId);
        //
        //        //设置子医院名称
        //        String subHospitalName = tranLog.getSubHospitalName();
        //        String subHosName = response.getHeader(CDSS_SUB_HOSPITAL_NAME);
        //        if (StringUtil.isNotBlank(subHosName)) {
        //            subHospitalName = subHosName;
        //            response.setHeader(CDSS_SUB_HOSPITAL_NAME, "");
        //        }
        //        tranLog.setSubHospitalName(subHospitalName);
        //
        //        //设置子医院编码
        //        String subHospitalCode = tranLog.getSubHospitalCode();
        //        String subHosCode = response.getHeader(CDSS_SUB_HOSPITAL_CODE);
        //        if (StringUtil.isNotBlank(subHosCode)) {
        //            subHospitalCode = subHosCode;
        //            response.setHeader(CDSS_SUB_HOSPITAL_CODE, "");
        //        }
        //        tranLog.setSubHospitalCode(subHospitalCode);
    }


    /**
     * 增加入参
     *
     * @param o
     */
    public static void addBizReq(Object o) {
        handleRespHeaderMap(CDSS_REQ, o);
    }

    /**
     * 增加出参
     *
     * @param o
     */
    public static void addBizResp(Object o) {
        handleRespHeaderMap(CDSS_RESP, o);
    }

    //    /**
    //     * 设置医院id
    //     *
    //     * @param o
    //     */
    //    public static void addBizHospitalId(Object o) {
    //        handleRespHeaderMap(CDSS_HOSPITAL_ID, o);
    //    }
    //
    //    /**
    //     * 设置子医院编码
    //     *
    //     * @param o
    //     */
    //    public static void addBizSubHospitalName(Object o) {
    //        handleRespHeaderMap(CDSS_SUB_HOSPITAL_NAME, o);
    //    }
    //
    //    /**
    //     * 设置子医院名称
    //     *
    //     * @param o
    //     */
    //    public static void addBizSubHospitalCode(Object o) {
    //        handleRespHeaderMap(CDSS_SUB_HOSPITAL_CODE, o);
    //    }
    //
    //    /**
    //     * 设置病人标识
    //     *
    //     * @param o
    //     */
    //    public static void addBizPatientId(Object o) {
    //        handleRespHeaderMap(CDSS_PATIENT_ID, o);
    //    }


    private static void handleRespHeaderMap(String key, Object o) {
        HttpServletResponse response = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getResponse();
        String value = response.getHeader(key);
        if (StringUtil.isNotBlank(value)) {
            value += CDSS_LOG_SP + FastJsonUtils.getBeanToJson(o);
        } else {
            value = FastJsonUtils.getBeanToJson(o);
        }
        // 设置Header
        response.setHeader(key, value);
    }
}